#include "mfPicoDstReader.h"
#include "mfNamespace.h"
#include "mfProcessEvent.h"
#include "mfInputReader.h"
#include "mfSelector.h"

#include <TStopwatch.h>
#include <TProfile.h>
#include <TClonesArray.h>
#include <TFile.h>
#include <iostream>
#include <TMath.h>

#include <PicoDstMCEvent.h>
#include <PicoDstRecoEvent.h>
#include <PicoDstMCTrack.h>
#include <PicoDstRecoTrack.h>
#include <PicoDstFHCal.h>

using mfdca::Ndim;
using mfEp::EpNames;
using mfEp::NetaGaps;
using mfEp::TpcEtaGap;
using mfEp::TpcLEp;
using mfEp::TpcREp;
using mfEp::EpNames::kFHCal;
using mfEp::EpNames::kFHCal_L;
using mfEp::EpNames::kFHCal_R;
using mfRes::NresBin;

using TMath::ATan2;
using TMath::Cos;
using TMath::Pi;
using TMath::Sin;

int main(int argc, char **argv)
{
  TString iFileName, oFileName, dcaFileName;

  if (argc < 7)
  {
    std::cerr << "./get-res -i INPUTFILE/LIST -o OUTPUT.root -dca DCA_FIT.root" << std::endl;
    return 1;
  }
  for (int i = 1; i < argc; i++)
  {
    if (std::string(argv[i]) != "-i" &&
        std::string(argv[i]) != "-o" &&
        std::string(argv[i]) != "-dca")
    {
      std::cerr << "\n[ERROR]: Unknown parameter " << i << ": " << argv[i] << std::endl;
      return 2;
    }
    else
    {
      if (std::string(argv[i]) == "-i" && i != argc - 1)
      {
        iFileName = argv[++i];
        continue;
      }
      if (std::string(argv[i]) == "-i" && i == argc - 1)
      {
        std::cerr << "\n[ERROR]: Input file name was not specified " << std::endl;
        return 1;
      }
      if (std::string(argv[i]) == "-o" && i != argc - 1)
      {
        oFileName = argv[++i];
        continue;
      }
      if (std::string(argv[i]) == "-o" && i == argc - 1)
      {
        std::cerr << "\n[ERROR]: Output file name was not specified " << std::endl;
        return 1;
      }
      if (std::string(argv[i]) == "-dca" && i != argc - 1)
      {
        dcaFileName = argv[++i];
        continue;
      }
      if (std::string(argv[i]) == "-dca" && i == argc - 1)
      {
        std::cerr << "\n[ERROR]: Input file name was not specified " << std::endl;
        return 1;
      }
    }
  }

  TStopwatch timer;
  timer.Start();

  mfPicoDstReader *input = new mfPicoDstReader();
  TChain *inChain = (TChain *)input->Open(iFileName);

  if (!inChain)
  {
    std::cerr << "[ERROR]: nothing to read. Maybe the input TTree(s) is(are) empty?" << std::endl;
    return 10;
  }

  // Init output profiles
  TProfile *pResRecoSqrTpc[NetaGaps];
  TProfile *pResRpTpc[NetaGaps];
  TProfile *pResRpTpcR[NetaGaps];
  TProfile *pResRpTpcL[NetaGaps];
  TProfile *pResRecoSqrFHCal;
  TProfile *pResV1RpFHCal;
  TProfile *pResV1RpFHCalR;
  TProfile *pResV1RpFHCalL;
  TProfile *pResV2RpFHCal;
  TProfile *pResV2RpFHCalR;
  TProfile *pResV2RpFHCalL;
  for (int i = 0; i < NetaGaps; i++)
  {
    pResRecoSqrTpc[i] = new TProfile(Form("pResRecoSqrtTpc_gap%i", i),
                                     Form("pResRecoSqrtTpc_gap%i", i),
                                     NresBin, 0., 100.);
    pResRpTpc[i] = new TProfile(Form("pResRpTpc_gap%i", i),
                                Form("pResRpTpc_gap%i", i),
                                NresBin, 0, 100);
    pResRpTpcR[i] = new TProfile(Form("pResRpTpcR_gap%i", i),
                                 Form("pResRpTpcR_gap%i", i),
                                 NresBin, 0, 100);
    pResRpTpcL[i] = new TProfile(Form("pResRpTpcL_gap%i", i),
                                 Form("pResRpTpcL_gap%i", i),
                                 NresBin, 0, 100);
  }
  pResRecoSqrFHCal = new TProfile(Form("pResRecoSqrFHCal"),
                                  Form("pResRecoSqrFHCal"),
                                  NresBin, 0., 100.);
  pResV1RpFHCal = new TProfile(Form("pResV1RpFHCal"),
                               Form("pResV1RpFHCal"),
                               NresBin, 0., 100.);
  pResV2RpFHCal = new TProfile(Form("pResV2RpFHCal"),
                               Form("pResV2RpFHCal"),
                               NresBin, 0., 100.);
  pResV1RpFHCalR = new TProfile(Form("pResV1RpFHCalR"),
                                Form("pResV1RpFHCalR"),
                                NresBin, 0., 100.);
  pResV1RpFHCalL = new TProfile(Form("pResV1RpFHCalL"),
                                Form("pResV1RpFHCalL"),
                                NresBin, 0., 100.);
  pResV2RpFHCalR = new TProfile(Form("pResV2RpFHCalR"),
                                Form("pResV2RpFHCalR"),
                                NresBin, 0., 100.);
  pResV2RpFHCalL = new TProfile(Form("pResV2RpFHCalL"),
                                Form("pResV2RpFHCalL"),
                                NresBin, 0., 100.);

  PicoDstMCEvent *mcEvent = nullptr;
  // PicoDstRecoEvent *recoEvent = nullptr;
  TClonesArray *recoTracks = nullptr;
  // TClonesArray *mcTracks = nullptr;
  TClonesArray *fhcalmodules = nullptr;

  inChain->SetBranchAddress("mcevent.", &mcEvent);
  // inChain->SetBranchAddress("recoevent.", &recoEvent);
  // inChain->SetBranchAddress("mctracks", &mcTracks);
  inChain->SetBranchAddress("recotracks", &recoTracks);
  inChain->SetBranchAddress("FHCalModules", &fhcalmodules);

  // Start event loop
  int n_entries = inChain->GetEntries();
  mfSelector *selector = new mfSelector();
  mfInputReader *inputReader = new mfInputReader();
  inputReader->SetDcaFitFile(dcaFileName);
  for (int iEv = 0; iEv < n_entries; iEv++)
  {
    if (iEv % 1000 == 0)
      std::cout << "Event [" << iEv << "/" << n_entries << "]" << std::endl;
    inChain->GetEntry(iEv);

    mfProcessEvent *procEvent = new mfProcessEvent();
    float cent = procEvent->GetCentB(mcEvent);
    procEvent->SetInputReader(inputReader);

    // std::cout << "\tNtracks = " << recoTracks->GetEntriesFast() << std::endl;

    mfEventInfo evInfo = procEvent->ProcessRecoEP(recoTracks, fhcalmodules);
    if (evInfo.GetQvectors().size() == 0)
      continue;

    for (int iGap = 0; iGap < NetaGaps; iGap++)
    {
      // Ntracks >= 4 for TpcR and TpcL EP
      if (!selector->isGoodTpcEp(evInfo.GetEpMult(TpcREp[iGap]), evInfo.GetEpMult(TpcLEp[iGap])))
        continue;

      double PsiR = evInfo.GetQvector(TpcREp[iGap]).GetPhi();
      double PsiL = evInfo.GetQvector(TpcLEp[iGap]).GetPhi();
      double resReco = Cos(2. * (PsiR - PsiL));
      double resRpR = Cos(2. * (PsiR - mcEvent->GetPhiRP()));
      double resRpL = Cos(2. * (PsiL - mcEvent->GetPhiRP()));

      pResRecoSqrTpc[iGap]->Fill(cent, resReco);
      pResRpTpcR[iGap]->Fill(cent, resRpR);
      pResRpTpcL[iGap]->Fill(cent, resRpL);
      pResRpTpc[iGap]->Fill(cent, resRpR);
      pResRpTpc[iGap]->Fill(cent, resRpL);
    }

    // FHCal energy > 0 in FHCal_L and FHCal_R
    if (!selector->isGoodFHCalEp(evInfo.GetEpWeight(kFHCal_R), evInfo.GetEpWeight(kFHCal_R)))
      continue;

    double PsiFHCalR = evInfo.GetQvector(kFHCal_R).GetPhi();
    double PsiFHCalL = evInfo.GetQvector(kFHCal_L).GetPhi();
    double PsiFHCal = evInfo.GetQvector(kFHCal).GetPhi();
    // double qx = evInfo.GetQvector(kFHCal_R).X() + evInfo.GetQvector(kFHCal_L).X();
    // double qy = evInfo.GetQvector(kFHCal_R).Y() + evInfo.GetQvector(kFHCal_L).Y();
    // double PsiFHCal = ATan2(qy, qx) / evInfo.GetQvector(kFHCal).GetHarmonic();
    double resEpFHCal = Cos(1. * (PsiFHCalR - PsiFHCalL));
    double resV1RpFHCal = Cos(1. * (PsiFHCal - mcEvent->GetPhiRP()));
    double resV1RpFHCalR = Cos(1. * (PsiFHCalR - mcEvent->GetPhiRP()));
    double resV1RpFHCalL = Cos(1. * (PsiFHCalL - mcEvent->GetPhiRP()));
    double resV2RpFHCal = Cos(2. * (PsiFHCal - mcEvent->GetPhiRP()));
    double resV2RpFHCalR = Cos(2. * (PsiFHCalR - mcEvent->GetPhiRP()));
    double resV2RpFHCalL = Cos(2. * (PsiFHCalL - mcEvent->GetPhiRP()));

    pResRecoSqrFHCal->Fill(cent, resEpFHCal);
    pResV1RpFHCal->Fill(cent, resV1RpFHCal);
    pResV1RpFHCalR->Fill(cent, resV1RpFHCalR);
    pResV1RpFHCalL->Fill(cent, resV1RpFHCalL);
    pResV2RpFHCal->Fill(cent, resV2RpFHCal);
    pResV2RpFHCalR->Fill(cent, resV2RpFHCalR);
    pResV2RpFHCalL->Fill(cent, resV2RpFHCalL);
  }

  // Init output file
  TFile *fo = new TFile(oFileName.Data(), "recreate");
  fo->cd();
  for (int iGap = 0; iGap < NetaGaps; iGap++)
  {
    pResRecoSqrTpc[iGap]->Write();
    pResRpTpcR[iGap]->Write();
    pResRpTpcL[iGap]->Write();
    pResRpTpc[iGap]->Write();
  }
  pResRecoSqrFHCal->Write();
  pResV1RpFHCal->Write();
  pResV1RpFHCalR->Write();
  pResV1RpFHCalL->Write();
  pResV2RpFHCal->Write();
  pResV2RpFHCalR->Write();
  pResV2RpFHCalL->Write();
  fo->Close();

  timer.Stop();
  timer.Print();
  return 0;
}