#ifndef MPD_FLOW_EVENTINFO_H
#define MPD_FLOW_EVENTINFO_H

#include <iostream>
#include <vector>
#include <Rtypes.h>
#include <TString.h>

#include "mfQvector.h"

class mfEventInfo
{
private:
  std::pair<Float_t,Float_t> fCentRange;
  Int_t fMult;
  std::vector<mfQvector> fQvectors;
  std::vector<int> fEpMult;
  std::vector<double> fEpWeight;
public:
  mfEventInfo(/* args */);
  virtual ~mfEventInfo();

  // Setters
  void SetCentralityRange(Float_t centMin, Float_t centMax);
  void AddQv(mfQvector qv) { fQvectors.push_back(qv); }
  void AddEpMult(int _a) { fEpMult.push_back(_a); }
  void AddEpWeight(float _a) { fEpWeight.push_back(_a); }
  void SetQv(std::vector<mfQvector> _a) { fQvectors = _a; }
  void SetEpMult(std::vector<int> _a) { fEpMult = _a; }
  void SetEpWeight(std::vector<double> _a) { fEpWeight = _a; }
  void SetMultiplicity(Int_t _a) { fMult = _a; }

  // Getters
  std::pair<Float_t,Float_t> GetCentralityRange() const { return fCentRange; }
  Float_t GetCentralityMean() const { return 0.5*(fCentRange.second - fCentRange.first); }
  std::vector<mfQvector> GetQvectors() const { return fQvectors; }
  std::vector<int> GetEpMult() const { return fEpMult; }
  std::vector<double> GetEpWeight() const { return fEpWeight; }
  mfQvector GetQvector(Int_t i) const { return fQvectors.at(i); }
  int GetEpMult(int i) const { return fEpMult.at(i); }
  float GetEpWeight(int i) const { return fEpWeight.at(i); }
  Int_t GetMultiplicity() const { return fMult; }

  ClassDef(mfEventInfo,0);
};

#endif