#include <map>
#include <iostream>

#include <TString.h>
#include <TMath.h>

#include "mfProcessEvent.h"
#include "mfQvector.h"
#include "mfSelector.h"

using mfcent::CentB;
using mfdca::NsigmaDcaCut;
using mfEp::EpNames;
using mfEp::GetFHCalPhi;
using mfEp::NetaGaps;
using mfEp::Nmodules;
using mfEp::NsubEvents;
using mfEp::NsubEventsTpc;
using mfEp::TpcEtaGap;
using mfEp::TpcLEp;
using mfEp::TpcREp;
using mfEp::EpNames::kFHCal;
using mfEp::EpNames::kFHCal_L;
using mfEp::EpNames::kFHCal_R;
using mfEp::BadFHCalModules;

using TMath::Abs;
using TMath::Cos;
using TMath::Sin;

ClassImp(mfProcessEvent);

mfProcessEvent::mfProcessEvent(/* args */)
{
}

mfProcessEvent::~mfProcessEvent()
{
}

Float_t mfProcessEvent::GetCentB(PicoDstMCEvent *const &mcEvent)
{
  return CentB(mcEvent->GetB());
}

mfEventInfo mfProcessEvent::ProcessRecoEP(TClonesArray *const &recoTracks, TClonesArray *const &fhcalModules)
{
  int mult = 0;
  std::vector<mfQvector> Qvectors;
  std::vector<double> Qx, Qy, Wt;
  std::vector<int> EpMult;

  mfSelector *selector = new mfSelector();

  if (!recoTracks)
    return fEventInfo;
  if (recoTracks->GetEntriesFast() == 0)
    return fEventInfo;
  if (!fInputReader)
    return fEventInfo;

  for (int i = 0; i < NsubEvents; i++)
  {
    Qx.push_back(0.);
    Qy.push_back(0.);
    Wt.push_back(0.);
    EpMult.push_back(0);
  }

  for (int iTrack = 0; iTrack < recoTracks->GetEntriesFast(); iTrack++)
  {
    auto recoTrack = (PicoDstRecoTrack *)recoTracks->UncheckedAt(iTrack);
    if (!recoTrack)
      continue;
    
    // Get refMult
    if (selector->isRefMult(recoTrack))
    {
      mult++;
    }

    // Primary track selection
    if (!selector->isPrimary(recoTrack->GetDCAx(), fInputReader->GetDcaSigmaX(recoTrack)))
      continue;
    if (!selector->isPrimary(recoTrack->GetDCAy(), fInputReader->GetDcaSigmaY(recoTrack)))
      continue;
    if (!selector->isPrimary(recoTrack->GetDCAz(), fInputReader->GetDcaSigmaZ(recoTrack)))
      continue;

    // Good reco track selection
    if (!selector->isGoodRecoTrack(recoTrack))
      continue;

    for (int iGap = 0; iGap < NetaGaps; iGap++)
    {
      // Tpc left
      if (recoTrack->GetEta() < -1. * TpcEtaGap[iGap])
      {
        Qx.at(TpcLEp[iGap]) += recoTrack->GetPt() * Cos(2. * recoTrack->GetPhi());
        Qy.at(TpcLEp[iGap]) += recoTrack->GetPt() * Sin(2. * recoTrack->GetPhi());
        Wt.at(TpcLEp[iGap]) += recoTrack->GetPt();
        EpMult.at(TpcLEp[iGap]) += 1;
      }
      // Tpc right
      if (recoTrack->GetEta() > TpcEtaGap[iGap])
      {
        Qx.at(TpcREp[iGap]) += recoTrack->GetPt() * Cos(2. * recoTrack->GetPhi());
        Qy.at(TpcREp[iGap]) += recoTrack->GetPt() * Sin(2. * recoTrack->GetPhi());
        Wt.at(TpcREp[iGap]) += recoTrack->GetPt();
        EpMult.at(TpcREp[iGap]) += 1;
      }
    }
  }
  for (int i = 0; i < NsubEventsTpc; i++)
  {
    if (Wt.at(i) == 0 || EpMult.at(i) == 0)
    {
      // std::cerr << "No tracks for EP" << std::endl;
      return fEventInfo;
    }
    else
    {
      Qx.at(i) /= Wt.at(i);
      Qy.at(i) /= Wt.at(i);
      Qvectors.push_back(mfQvector(2., Qx.at(i), Qy.at(i)));
      // std::cout << "Qx[" << i << "] = " << Qx.at(i) << ", "
      //           << "Qy[" << i << "] = " << Qy.at(i) << std::endl;
    }
  }

  for (int iMod = 0; iMod < fhcalModules->GetEntriesFast(); iMod++)
  {
    auto fhcalModule = (PicoDstFHCal *)fhcalModules->UncheckedAt(iMod);
    if (!fhcalModule)
      continue;
    if ((iMod == BadFHCalModules.at(0)) || (iMod == BadFHCalModules.at(1)))
      continue;
    double phi = GetFHCalPhi(iMod);

    // std::cout << "\tModule " << iMod
    //           << ": phi " << phi
    //           << ", Energy " << fhcalModule->GetEnergy() << std::endl;

    int w_sign = (iMod < Nmodules) ? 1 : -1;
    // for left FHCal modules
    if (iMod < Nmodules)
    {
      Qx.at(kFHCal_L) += w_sign * fhcalModule->GetEnergy() * Cos(1. * phi);
      Qy.at(kFHCal_L) += w_sign * fhcalModule->GetEnergy() * Sin(1. * phi);
      Wt.at(kFHCal_L) += fhcalModule->GetEnergy();
    }
    // for right FHCal modules
    if (iMod >= Nmodules)
    {
      Qx.at(kFHCal_R) += w_sign * fhcalModule->GetEnergy() * Cos(1. * phi);
      Qy.at(kFHCal_R) += w_sign * fhcalModule->GetEnergy() * Sin(1. * phi);
      Wt.at(kFHCal_R) += fhcalModule->GetEnergy();
    }
    // Total FHCal modules
    Qx.at(kFHCal) += w_sign * fhcalModule->GetEnergy() * Cos(1. * phi);
    Qy.at(kFHCal) += w_sign * fhcalModule->GetEnergy() * Sin(1. * phi);
    Wt.at(kFHCal) += fhcalModule->GetEnergy();
  }

  for (int i = NsubEventsTpc; i < NsubEvents; i++)
  {
    if (Wt.at(i) == 0)
    {
      // std::cerr << "No tracks for EP" << std::endl;
      return fEventInfo;
    }
    else
    {
      Qx.at(i) /= Wt.at(i);
      Qy.at(i) /= Wt.at(i);
      Qvectors.push_back(mfQvector(1., Qx.at(i), Qy.at(i)));
      // std::cout << "Qx[" << i << "] = " << Qx.at(i) << ", "
      //           << "Qy[" << i << "] = " << Qy.at(i) << std::endl;
    }
  }

  fEventInfo.SetMultiplicity(mult);
  fEventInfo.SetQv(Qvectors);
  fEventInfo.SetEpMult(EpMult);
  fEventInfo.SetEpWeight(Wt);

  return fEventInfo;
}