#include "mfRecoTrackSelector.h"
#include "mfNamespace.h"

using mfcent::etaRefMultCut;
using mfcent::ptRefMultCut;
using mfdca::NsigmaDcaCut;
using mfEp::MinFHCalEpEnergy;
using mfEp::MinTpcEpMult;
using mfFlow::MinV1PtEta;
using mfPid::MinPidProbability;
using mfTrack::MaxTrackEta;
using mfTrack::MaxTrackPt;
using mfTrack::MinTrackNhits;

ClassImp(mfRecoTrackSelector);

mfRecoTrackSelector::mfRecoTrackSelector(/* args */)
{
}

mfRecoTrackSelector::~mfRecoTrackSelector()
{
}

Bool_t mfRecoTrackSelector::isRefMult(PicoDstRecoTrack *const &track)
{
  if (track->GetPt() >= ptRefMultCut && abs(track->GetEta()) < etaRefMultCut)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isGoodRecoTrack(PicoDstRecoTrack *const &track)
{
  if (track->GetPt() < MaxTrackPt &&
      track->GetNhits() > MinTrackNhits &&
      abs(track->GetEta()) < MaxTrackEta)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isPrimary(Double_t TrackDca, Double_t DcaSigma)
{
  if (abs(TrackDca) < NsigmaDcaCut * DcaSigma)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isGoodTpcEp(Int_t multTpc_R, Int_t multTpc_L)
{
  if (multTpc_R >= MinTpcEpMult && multTpc_L >= MinTpcEpMult)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isGoodFHCalEp(Float_t EnFHCal_R, Float_t EnFHCal_L)
{
  if (EnFHCal_R > MinFHCalEpEnergy && EnFHCal_L > MinFHCalEpEnergy)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isGoodForV1Pt(Float_t eta)
{
  if (abs(eta) > MinV1PtEta && abs(eta) < MaxTrackEta)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isPion(PicoDstRecoTrack *const &track)
{
  if (track->GetPidProbPion() >= MinPidProbability)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isKaon(PicoDstRecoTrack *const &track)
{
  if (track->GetPidProbKaon() >= MinPidProbability)
    return true;
  else
    return false;
}

Bool_t mfRecoTrackSelector::isProton(PicoDstRecoTrack *const &track)
{
  if (track->GetPidProbProton() >= MinPidProbability)
    return true;
  else
    return false;
}