#include "mfNamespace.h"
#include <TMath.h>
#include <TVector2.h>
#include <Math/SpecFuncMathMore.h>

using ROOT::Math::cyl_bessel_i;
using TMath::ATan2;
using TMath::Cos;
using TMath::Sin;
using TMath::Sqrt;
using TMath::Pi;

int mfdca::GetPtBin(float pt)
{
  int pt_bin = -1;
  for (int i = 0; i < NptBins; ++i)
    if ((pt > ptBins[i]) && (pt <= ptBins[i + 1]))
      pt_bin = i;
  return pt_bin;
}

int mfdca::GetEtaBin(float eta)
{
  int eta_bin = -1;
  for (int i = 0; i < NetaBins; ++i)
    if ((eta > etaBins[i]) && (eta <= etaBins[i + 1]))
      eta_bin = i;
  return eta_bin;
}

float mfcent::CentB(float bimp)
{
  // Hard coded centrality defenition
  // based on the impact parameter
  float fcent;
  if (bimp < 4.18)
    fcent = 0; // 0-10%
  else if (bimp < 6.01)
    fcent = 10; //10-20%
  else if (bimp < 7.37)
    fcent = 20; //20-30%
  else if (bimp < 8.52)
    fcent = 30; //30-40%
  else if (bimp < 9.57)
    fcent = 40; //40-50%
  else if (bimp < 10.55)
    fcent = 50; //50-60%
  else if (bimp < 11.46)
    fcent = 60; //60-70%
  else if (bimp < 12.31)
    fcent = 70; //70-80%
  else
    fcent = -1;

  return fcent + 5.;
}

// bool mfcent::isRefMult(float pt, float eta)
// {
//   bool result = (pt > ptRefMultCut && abs(eta) < etaRefMultCut) ? true : false;
//   return result;
// }

double mfEp::GetFHCalPhi(int iModule)
{
  int xAxisSwitch = (iModule < Nmodules) ? 1 : -1;
  int module = (iModule < Nmodules) ? iModule : iModule - Nmodules;
  double x, y, phi = -999.;
  if (module >= 0 && module <= 4)
  {
    y = 45.;
    x = (module - 2) * 15.;
    phi = ATan2(y, x * xAxisSwitch);
  }
  else if ((module >= 5) && (module <= 39))
  {
    y = (3 - (module + 2) / 7) * 15.;
    x = (3 - (module + 2) % 7) * 15.;
    phi = ATan2(y, x * xAxisSwitch);
  }
  else if ((module >= 40) && (module <= 44))
  {
    y = -45.;
    x = (module - 42) * 15.;
    phi = ATan2(y, x * xAxisSwitch);
  }

  return phi;
}

float mfEp::RangeAngle_02Pi(float phi)
{
  TVector2 vect;
  vect.Set(Cos(phi), Sin(phi));
  return vect.Phi();
}

float mfEp::RangeAngle(float phi)
{
  return ATan2(Sin(phi), Cos(phi));
}

double mfRes::ResEventPlane(double chi, double harmonic)
{
  Double_t con = Sqrt(Pi() / 2.) / 2.;
  Double_t arg = chi * chi / 4.;
  Double_t res = con * chi * exp(-arg) * (cyl_bessel_i(0.5 * (harmonic - 1), arg) + cyl_bessel_i(0.5 * (harmonic + 1), arg));
  return res;
}

double mfRes::ResChi(double res, double harmonic, int Niter)
{
  Double_t chi = 2.0;
  Double_t delta = 1.0;
  for (int i = 0; i < NresChiIter; i++)
  {
    if (ResEventPlane(chi, harmonic) < res)
    {
      chi = chi + delta;
    }
    else
    {
      chi = chi - delta;
    }
    delta = delta / 2.;
  }

  return chi;
}