#ifndef MPD_FLOW_NAMESPACE_H
#define MPD_FLOW_NAMESPACE_H

#include <vector>

namespace mfdca
{
  const float ptBins[] = {0., 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.5, 3.};
  const int NptBins = 12;

  const float etaBins[] = {-1.5, -1.2, -1., -0.8, -0.6, -0.4, -0.2, 0., 0.2, 0.4, 0.6, 0.8, 1., 1.2, 1.5};
  const int NetaBins = 14;

  const int Ndim = 3;

  const int NdcaFitIter = 3;

  const float NsigmaDcaCut = 2.;

  int GetPtBin(float pt);
  int GetEtaBin(float eta);
}; // namespace mfdca

namespace mfcent
{
  const float ptRefMultCut = 0.15;
  const float etaRefMultCut = 0.5;

  float CentB(float bimp);
  // bool isRefMult(float pt, float eta);
}; // namespace mfcent

namespace mfEp
{
  const int NetaGaps = 3;
  enum EpNames
  {
    kTpc_R_EtaGap1,
    kTpc_R_EtaGap2,
    kTpc_R_EtaGap3,
    kTpc_L_EtaGap1,
    kTpc_L_EtaGap2,
    kTpc_L_EtaGap3,
    kFHCal_R,
    kFHCal_L,
    kFHCal
  };
  const int NsubEvents = kFHCal + 1;
  const int NsubEventsTpc = kTpc_L_EtaGap3 + 1;
  const float TpcEtaGap[NetaGaps] = {0.05, 0.1, 0.25};
  const int TpcREp[NetaGaps] = {kTpc_R_EtaGap1, kTpc_R_EtaGap2, kTpc_R_EtaGap3};
  const int TpcLEp[NetaGaps] = {kTpc_L_EtaGap1, kTpc_L_EtaGap2, kTpc_L_EtaGap3};

  const int Nmodules = 45;
  double GetFHCalPhi(int iModule);
  float RangeAngle_02Pi(float phi); // make it from 0 to 2*Pi
  float RangeAngle(float phi);      // make it from -Pi to Pi

  const int MinTpcEpMult = 4;
  const float MinFHCalEpEnergy = 0.;
  const std::vector<int> BadFHCalModules = {22, 67};
}; // namespace mfEp

namespace mfRes
{
  const int NresBin = 10;
  const int NresChiIter = 50;
  double ResEventPlane(double chi, double harmonic);
  double ResChi(double res, double harmonic, int Niter);
}; // namespace mfRes

namespace mfTrack
{
  const float MaxTrackPt = 3.;
  const int MinTrackNhits = 32;
  const float MaxTrackEta = 1.5;
}; // namespace mfTrack

namespace mfPid
{
  const float MinPidProbability = 0.9;
  const int Npid = 4;
  const float Mass2 [] = {0.88035,0.2437,0.01948};
}; // namespace mfPid

namespace mfFlow
{
  const float MinV1PtEta = 0.2;
}; // namespace mfFlow

#endif